@@ -55,11 +55,6 @@
 #include "icalmemory.h"
 #include "icalparser.h"
 
-#ifdef WIN32
-#define HAVE_CTYPE_H
-#define HAVE_ISWSPACE
-#endif
-
 #ifdef HAVE_WCTYPE_H
 # include <wctype.h>
 /* Some systems have an imcomplete implementation on wctype (FreeBSD,
@@ -224,10 +219,7 @@ static char* make_segment(char* start, char* end)
     char *buf, *tmp;
     size_t size = (size_t)end - (size_t)start;
     
-    /*buf = icalmemory_tmp_buffer(size+1);*/
-    
     buf = icalmemory_new_buffer(size+1);
-
     strncpy(buf,start,size);
     *(buf+size) = 0;
 
@@ -287,6 +279,7 @@ char* parser_get_param_name(char* line, char **end, char **buf)
         *end = *end+1;
 	    next = parser_get_next_char('"',*end,0);
 	    if (next == 0) {
+			free(str);
 		    return 0;
 	    }
 
@@ -883,9 +876,9 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	str = parser_get_next_parameter(end,&end);
 	strstriplt(str);
 	if (str != 0){
-	    char* name;
-	    char* pvalue;
-		char *buf_value = NULL;
+	    char* name = 0;
+	    char* pvalue = 0;
+	    char *buf_value = NULL;
         
 	    icalparameter *param = 0;
 	    icalparameter_kind kind;
@@ -933,9 +926,26 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 			     ICAL_XLICERRORTYPE_PARAMETERNAMEPARSEERROR);
 		tail = 0;
 		parser->state = ICALPARSER_ERROR;
+		/* if (pvalue) {
+			free(pvalue);
+			pvalue = 0;
+		} */
+		if (name) {
+			free(name);
+			name = 0;
+		}
 		return 0;
 	    }
 
+	    /* if (pvalue) {
+		free(pvalue);
+		pvalue = 0;
+	    } */
+	    if (name) {
+		free(name);
+		name = 0;
+	    }
+
 	    if (param == 0){
 		/* 'tail' defined above */
 		insert_error(tail,str,"Cant parse parameter value",
@@ -972,7 +982,6 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 			tail, str, 
 			"Got a VALUE parameter with an unknown type",
 			ICAL_XLICERRORTYPE_PARAMETERVALUEPARSEERROR);
-		    icalparameter_free(param);
 			
 		    value_kind = 
 			icalproperty_kind_to_value_kind(
@@ -1094,11 +1103,6 @@ icalcomponent* icalparser_add_line(icalparser* parser,
 	    str = NULL;
 
 	} else {
-		if (str != NULL) {
-			icalmemory_free_buffer(str);
-			str = NULL;
-		}
-			
 	    if (vcount == 0){
 		char temp[200]; /* HACK */
 		
