@@ -167,14 +167,17 @@ icalcomponent* icalvcal_convert_with_defaults (VObject *object,
 {
 
    char* name =  (char*)vObjectName(object);
-   icalcomponent* container = icalcomponent_new(ICAL_XROOT_COMPONENT);
+   icalcomponent* container;
    icalcomponent* root;
    icalproperty *prop;
 
    icalerror_check_arg_rz( (object!=0),"Object");
 
+   container = icalcomponent_new(ICAL_XROOT_COMPONENT);
+
    /* The root object must be a VCALENDAR */
    if(*name==0 || strcmp(name,VCCalProp) != 0){
+       icalcomponent_free(container);
        return 0; /* HACK. Should return an error */
    }
 
@@ -357,6 +360,7 @@ static int get_alarm_properties (icalcomponent *comp, VObject *object,
 		attach = icalattach_new_from_url (s);
 		attach_prop = icalproperty_new_attach (attach);
 		icalcomponent_add_property (comp, attach_prop);
+		icalattach_unref(attach);
 
 		/* We output a "application/binary" FMTTYPE for Procedure
 		   alarms. */
@@ -468,6 +472,7 @@ static int get_alarm_properties (icalcomponent *comp, VObject *object,
 
 		fmttype_param = icalparameter_new_fmttype (defaults->alarm_audio_fmttype);
 		icalproperty_add_parameter (attach_prop, fmttype_param);
+		icalattach_unref(attach);
 	    } else {
 	        is_valid_alarm = 0;
 	    }
@@ -541,6 +546,7 @@ static int get_alarm_properties (icalcomponent *comp, VObject *object,
 		free (new_url);
 
 		icalproperty_set_attach (attach_prop, new_attach);
+		icalattach_unref(attach);
 
 	    } else {
 		is_valid_alarm = 0;
@@ -859,7 +865,7 @@ static char* rrule_parse_duration (char *s, struct icalrecurrencetype *recur,
 
     /* It must be followed by whitespace or the end of the string.
        I'm not sure if anything else is allowed. */
-    if (*s != '\0' && *s != ' ' && *s != '\t') {
+    if (s && *s != '\0' && *s != ' ' && *s != '\t') {
 	*error_message = "Invalid Duration";
 	return NULL;
     }
